from djtool.review import View
from django.http import JsonResponse
from django.core.cache import cache
from django.conf import settings


class SignOutView(View):
    def post(self, request, *args, **kwargs):
        try:
            del request.session['login']
            cache.set('admin%s' % request.admin.unionuuid, 0)
            response = JsonResponse(self.msg(20000))
        except:
            response = JsonResponse(self.msg(50000))
        response.delete_cookie('_login', domain=settings.COOKIE_DOMAIN)
        return response



# coding: utf-8
from __future__ import unicode_literals
import sys

PY2 = True
if sys.version < '3':
    from future.builtins import str, int
    PY2 = False

import datetime
import json
import types
import pytz

from decimal import Decimal

from .TimeFormatFactory import TimeFormatFactory

try:
    from django.db import models
    from django.db.models import manager
    from django.core.paginator import Page
    from django.db.models.query import QuerySet
    from django.db.models.fields.files import ImageFieldFile, FileField
except ImportError:
    raise RuntimeError('django is required in django simple serializer')


class Serializer(object):
    include_attr = []
    exclude_attr = []
    objects = []
    origin_data = None
    output_type = 'raw'
    datetime_format = 'timestamp'
    foreign = False
    many = False
    through = True

    def __init__(self, data, datetime_format='timestamp', output_type='raw', include_attr=None, exclude_attr=None,
                 foreign=False, many=False, through=True, extra_attr=None, *args, **kwargs):
        if include_attr:
            self.include_attr = include_attr
        if exclude_attr:
            self.exclude_attr = exclude_attr
        self.extra_attr = extra_attr
        self.origin_data = data
        self.output_type = output_type
        self.foreign = foreign
        self.many = many
        self.through = through
        self.through_fields = []
        self.source_field = None
        self.datetime_format = datetime_format
        self.time_func = TimeFormatFactory.get_time_func(datetime_format)
        self._dict_check = kwargs.get('dict_check', False)

    def check_attr(self, attr):
        if self.exclude_attr and attr in self.exclude_attr:
            return False
        if self.include_attr and attr not in self.include_attr:
            return False
        return True

    def data_inspect(self, data, extra=None):
        if isinstance(data, (QuerySet, Page, list)):
            convert_data = []
            if extra:
                for i, obj in enumerate(data):
                    convert_data.append(self.data_inspect(obj, extra.get(
                        **{self.through_fields[0]: obj, self.through_fields[1]: self.source_field})))
            else:
                for obj in data:
                    convert_data.append(self.data_inspect(obj))
            return convert_data
        elif isinstance(data, models.Model):
            obj_dict = {}
            concrete_model = data._meta.concrete_model

            for field in concrete_model._meta.local_fields:
                if field.remote_field is None:
                    if self.check_attr(field.name) and hasattr(data, field.name):
                        obj_dict[field.name] = self.data_inspect(getattr(data, field.name))
                else:
                    if self.check_attr(field.name) and self.foreign:
                        obj_dict[field.name] = self.data_inspect(getattr(data, field.name))
            for field in concrete_model._meta.many_to_many:
                if self.check_attr(field.name) and self.many:
                    obj_dict[field.name] = self.data_inspect(getattr(data, field.name))
            for k, v in data.__dict__.items():
                if not str(k).startswith('_') and k not in obj_dict.keys() and self.check_attr(k):
                    obj_dict[k] = self.data_inspect(v)
            if extra:
                for field in extra._meta.concrete_model._meta.local_fields:
                    if field.name not in obj_dict.keys() and field.name not in self.through_fields:
                        if field.remote_field is None:
                            if self.check_attr(field.name) and hasattr(extra, field.name):
                                obj_dict[field.name] = self.data_inspect(getattr(extra, field.name))
                        else:
                            if self.check_attr(field.name) and self.foreign:
                                obj_dict[field.name] = self.data_inspect(getattr(extra, field.name))
            if self.extra_attr:
                for name in self.extra_attr:
                    obj_dict[name] = self.data_inspect(getattr(data, name))
            return obj_dict
        elif isinstance(data, manager.Manager):
            through_list = data.through._meta.concrete_model._meta.local_fields
            through_data = data.through._default_manager
            self.through_fields = [data.target_field.name, data.source_field.name]
            self.source_field = data.instance
            if len(through_list) > 3 and self.through:
                return self.data_inspect(data.all(), through_data)
            else:
                return self.data_inspect(data.all())
        elif isinstance(data, (datetime.datetime, datetime.date, datetime.time)):
            return data.replace(tzinfo=pytz.utc).astimezone(pytz.timezone('Asia/Shanghai')).strftime('%F %T')
            # return self.time_func(data)
        elif isinstance(data, (ImageFieldFile, FileField)):
            return data.url if data.url else data.path
        elif isinstance(data, Decimal):
            return float(data)
        elif isinstance(data, dict):
            obj_dict = {}
            if self._dict_check:
                for k, v in data.items():
                    obj_dict[k] = self.data_inspect(v)
            else:
                for k, v in data.items():
                    if self.check_attr(k):
                        obj_dict[k] = self.data_inspect(v)
            return obj_dict
        elif isinstance(data, (str, bool, float, int)):
            return data
        elif isinstance(data, types.MethodType):
            if isinstance(data(), (str, bool, float, int, list, dict, tuple)):
                return data()
            else:
                return None
        else:
            return None

    def data_format(self):
        self.objects = self.data_inspect(self.origin_data)

    def get_values(self):
        output_switch = {'dict': self.objects,
                         'raw': self.objects,
                         'json': json.dumps(self.objects, indent=4)}
        return output_switch.get(self.output_type, self.objects)

    def __call__(self):
        self.data_format()
        return self.get_values()


def serializer(data, datetime_format='timestamp', output_type='raw', include_attr=None, exclude_attr=None,
               foreign=False, many=False, through=True, *args, **kwargs):
    s = Serializer(data, datetime_format, output_type, include_attr, exclude_attr,
                   foreign, many, through, *args, **kwargs)
    return s()



from mns.account import Account
from django.conf import settings


class MNS:
    def __init__(self):
        c = ['MNS_ENDPOINT', 'MNS_ACCESSID', 'MNS_ACCESSKEY', 'MNS_TOPIC']
        for one in c:
            if not hasattr(settings, one):
                raise Exception('请配置%s参数' % one)
            else:
                setattr(self, one, getattr(settings, one))
        self.account = Account(self.MNS_ENDPOINT, self.MNS_ACCESSID, self.MNS_ACCESSKEY)
        self.topic = self.account.get_topic(self.MNS_TOPIC)

    # def sms(self, sign_name, template_code, single=False):
    #     try:
    #         self.sms = DirectSMSInfo(free_sign_name=sign_name, template_code=template_code, single=single)
    #         return True
    #     except:
    #         raise Exception('1')

    # def send(self, receiver, params):
    #     try:
    #         self.sms.add_receiver(receiver=receiver, params=params)
    #         msg = TopicMessage('mm', direct_sms=self.sms)
    #         self.topic.publish_message(msg)
    #         return True
    #     except:
    #         raise Exception('2')


mns = MNS()



# coding: utf-8

import time
import datetime
from functools import partial

try:
    from django.utils import timezone
except ImportError:
    raise RuntimeError('Django is required for django simple serializer.')


class TimeFormatFactory(object):
    def __init__(self):
        super(TimeFormatFactory, self).__init__()

    @staticmethod
    def datetime_to_string(datetime_time, time_format='%Y-%m-%d %H:%M:%S'):
        if isinstance(datetime_time, datetime.datetime):
            if datetime_time.tzinfo:
                datetime_time = datetime_time.astimezone(timezone.get_current_timezone())
            return datetime_time.strftime(time_format)
        elif isinstance(datetime_time, datetime.time):
            time_format = '%H:%M:%S'
        elif isinstance(datetime_time, datetime.date):
            time_format = '%Y-%m-%d'
        return datetime_time.strftime(time_format)

    @staticmethod
    def datetime_to_timestamp(datetime_time, time_format=None):
        if isinstance(datetime_time, datetime.datetime):
            if datetime_time.tzinfo:
                datetime_time = datetime_time.astimezone(timezone.get_current_timezone())
            return time.mktime(datetime_time.timetuple())
        return time.mktime(datetime_time.timetuple())

    @staticmethod
    def get_time_func(func_type='string'):
        if func_type == 'string':
            return TimeFormatFactory.datetime_to_string
        elif func_type == 'timestamp':
            return TimeFormatFactory.datetime_to_timestamp
        else:
            return TimeFormatFactory.datetime_to_string



from django.conf import settings
import requests
from djtool.common import Common
import json
import shortuuid
import time
import hmac
import base64
import hashlib


class ClientOauth(Common):

    def __init__(self, client_id='', client_secret=''):
        self.client_id = client_id or settings.CLIENT_ID
        self.client_secret = client_secret or settings.CLIENT_SECRET
        self.requests = requests.Session()

    def _login(self, user, data):
        data["client_id"] = self.client_id
        data["grant_type"] = 'password'
        response = self.requests.post('%s/api/v1/%s/token/' % (settings.ACCOUNT_API_HOST, user), data=data, headers={'X-CSRFToken': self.requests.cookies.get('csrftoken')}).json()
        return response.get('data', '') if response.get("code") == 20000 else ''

    def usertoken(self, data):
        return self._login('user', data)

    def admintoken(self, data):
        return self._login('admin', data)

    def info(self, token):
        r = self.requests.get('%s/api/v1/userinfo/' % settings.ACCOUNT_API_HOST, headers={"TOKEN": token}).json()
        return r.get('data', {}) if r.get('code') == 20000 else {}

    def client_token(self, timeout=30000):
        data = {}
        data['client_id'] = self.client_id
        data["salt"] = shortuuid.uuid()
        data["expires"] = time.time() + timeout
        payload = json.dumps(data).encode("utf8")
        sig = self._get_signature(payload)
        return self._encode_token_bytes(payload+sig)

    def servertoken(self, data={}):
        data["client_id"] = self.client_id
        data["grant_type"] = 'token'
        response = self.requests.post('%s/api/v1/token/?client_id=%s' % (settings.ACCOUNT_API_HOST, self.client_id), data=data, headers={'X-CSRFToken': self.requests.cookies.get('csrftoken'), 'TOKEN': self.get_token()}).json()
        return response.get('data', '') if response.get('code') == 20000 else ''

    def get_token(self, data={}, timeout=10):
        data = data.copy()
        if "salt" not in data:
            data["salt"] = shortuuid.uuid()
        if "expires" not in data:
            data["expires"] = time.time() + timeout
        payload = json.dumps(data).encode("utf8")
        sig = self._get_signature(payload)
        return self._encode_token_bytes(payload+sig).decode()

    def application(self):
        response = self.requests.get('%s/api/v1/application/?client_id=%s&env=%s' % (settings.ACCOUNT_API_HOST, self.client_id, settings.ENVIRONMENT), headers={'X-CSRFToken': self.requests.cookies.get('csrftoken'), 'TOKEN': self.get_token()}).json()
        return response.get('data', []) if response.get('code') == 20000 else []

    def _get_signature(self, value):
        return hmac.new(self.client_secret.encode("utf8"), value, hashlib.sha256).hexdigest().encode("utf8")

    def _encode_token_bytes(self, data):
        return base64.urlsafe_b64encode(data)

    def _update_user_info(self, user, uuid, data):
        response = self.requests.put('%s/api/v1/%s/%s/?client_id=%s' % (settings.ACCOUNT_API_HOST, user, uuid, self.client_id), data=data, headers={'TOKEN': self.get_token()}).json()
        return response.get('data', {}) if response.get('code') == 20000 else {}

    def update_admin_info(self, uuid, data):
        return self._update_user_info('admin', uuid, data)




from django.views.generic import View, TemplateView
from django.views.generic.base import TemplateResponseMixin
from django.views.generic.list import MultipleObjectMixin
from djtool.dss.Serializer import serializer
from django.http import JsonResponse, HttpResponse
from djtool.common import Common
from django.shortcuts import get_object_or_404
import importlib
from django.core.cache import cache
import urllib


class MultipleObjectMixin(MultipleObjectMixin):

    def get_context_data(self, **kwargs):
        if 'view' not in kwargs:
            kwargs['view'] = self
        return kwargs


class View(View, Common):

    def dispatch(self, request, *args, **kwargs):
        if request.method.lower() in self.http_method_names:
            handler = getattr(self, request.method.lower(), self.http_method_not_allowed)
        else:
            handler = self.http_method_not_allowed
        if hasattr(self, 'patch'):
            request, args, kwargs = self.patch(request, args, kwargs)
            if isinstance(request, HttpResponse):
                return request
        if hasattr(self, 'nav'):
            request, args, kwargs = self.nav(request, args, kwargs)
        if hasattr(self, 'power'):
            reqeust, args, kwargs = self.power(request, args, kwargs)
        if hasattr(self, 'permission'):
            request, args, kwargs = self.permission(request, args, kwargs)
            if isinstance(request, HttpResponse):
                return request
        if request.method == 'GET':
            kwargs["l"] = "{{"
            kwargs["r"] = "}}"
        return handler(request, *args, **kwargs)


class TemplateView(TemplateView, View):

    def get(self, request, *args, **kwargs):
        if hasattr(self, 'content') and isinstance(self.content, dict):
            for key, value in self.content.items():
                kwargs[key] = value
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context)


class ListView(TemplateResponseMixin, MultipleObjectMixin, View):

    def __get_result(self, request, *args, **kwargs):
        self.object_list = self.get_queryset()
        paginator = self.paginator if hasattr(self, 'paginator') else True
        if paginator:
            if hasattr(self, 'pre'):
                pre = self.pre
            elif request.GET.get('limit'):
                pre = self.request.GET.get('limit')
            else:
                pre = 10
            page = Common.page(self.object_list, request.page, pre=pre)
            self.object_list = page
        fomat = self.serializer if hasattr(self, 'serializer') else {}
        data = serializer(self.object_list, **fomat)
        if paginator:
            d = []
            i = 0
            uuids = []
            for row in data:
                if self.show_id:
                    i += 1
                    row['id'] = Common.ID_desc(page.paginator.count, request.page, i)
                d.append(row)
                uuids.append(row['uuid'])
            result = {"data": d, "total": page.paginator.num_pages, "count": page.paginator.count}
        else:
            result = {"data": data, "count": len(data)}
        return result

    def get(self, request, *args, **kwargs):
        if request.GET.get('curr'):
            request.page = request.GET.get('curr')
            if hasattr(self, 'cache') and self.cache:
                key = request.get_full_path()
                result = cache.get(key)
                if not result:
                    result = self.__get_result(request, *args, **kwargs)
                    cache.set(key, result, timeout=60 * 60)
            else:
                result = self.__get_result(request, *args, **kwargs)
            return JsonResponse(result)
        elif request.GET.get('page'):
            request.page = request.GET.get('page')
            if hasattr(self, 'cache') and self.cache:
                key = request.get_full_path()
                result = cache.get(key)
                if not result:
                    result = self.__get_result(request, *args, **kwargs)
                    cache.set(key, result, timeout=60 * 60)
            else:
                result = self.__get_result(request, *args, **kwargs)
            layui_table = {}
            layui_table['code'] = 0
            layui_table['msg'] = ""
            layui_table['count'] = result["count"]
            layui_table['data'] = result["data"]
            return JsonResponse(layui_table)
        else:
            if request.GET.get('uuid') and hasattr(self, 'model_class') and self.model_class:
                content_name = str(self.model_class._meta).split('.')[-1]
                kwargs[content_name] = get_object_or_404(self.model_class, uuid=request.GET.get('uuid'))
            if hasattr(self, 'content') and isinstance(self.content, dict):
                for key, value in self.content.items():
                    kwargs[key] = value
            context = self.get_context_data(**kwargs)
            return self.render_to_response(context)

    def put(self, request, *args, **kwargs):
        if not hasattr(request, 'data'):
            request.data = dict(urllib.parse.parse_qsl(request.body.decode()))
        if request.data.get('uuid') and hasattr(self, 'model_class') and self.model_class:
            try:
                object_one = self.model_class.objects.get(uuid=request.data.get('uuid'), del_state=1)
                form = self.form_class(request.data.copy(), instance=object_one)
                if form.is_valid():
                    if hasattr(self, 'method_put') and len(self.method_put) == 2:
                        method = self.method_put[0].split('.')
                        method_class = '.'.join(method[:-1])
                        action_class = getattr(importlib.import_module(method_class), method[-1])
                        class_object = action_class()
                        result = getattr(class_object, self.method_put[1])(request, form, *args, **kwargs)
                    else:
                        form.save()
                        result = self.msg(20002)
                        return JsonResponse(result)
            except:
                return JsonResponse(self.msg(50002))
        return JsonResponse(self.msg(list(form.errors.values())[0][0]))

    def delete(self, request, *args, **kwargs):
        if request.GET.get('uuid'):
            if hasattr(self, 'relation') and self.relation:
                query_delete = self.queryset.filter(uuid__in=request.GET.get('uuid', '').split(','))
                if isinstance(self.relation, str):
                    getattr(get_object_or_404(self.model_class, uuid=request.GET.get('id')), self.relation).remove(*query_delete)
                else:
                    getattr(get_object_or_404(self.model_class, uuid=request.GET.get('id')), str(query_delete.model._meta).split('.')[-1]).remove(*query_delete)
            else:
                self.queryset.filter(uuid__in=request.GET.get('uuid', '').split(',')).update(del_state=0)
            return JsonResponse(self.msg(20006))
        return JsonResponse(self.msg(50006))


class ActionView(TemplateView, Common):

    def get(self, request, *args, **kwargs):
        if request.GET.get('uuid') and hasattr(self, 'model_class') and self.model_class:
            content_name = self.__class__.__name__.lower().replace('view', '')
            kwargs[content_name] = get_object_or_404(self.model_class, uuid=request.GET.get('uuid'), del_state=1)
            kwargs['action'] = 'edit'
        else:
            kwargs['action'] = 'add'
        if hasattr(self, 'content') and isinstance(self.content, dict):
            for key, value in self.content.items():
                kwargs[key] = value
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context)

    def post(self, request, *args, **kwargs):
        data = request.data if hasattr(request, 'data') else request.POST
        form = self.form_class(data)
        if form.is_valid():
            if hasattr(self, 'method_post') and len(self.method_post) == 2:
                method = self.method_post[0].split('.')
                method_class = '.'.join(method[:-1])
                action_class = getattr(importlib.import_module(method_class), method[-1])
                class_object = action_class(admin=request.admin)
                result = getattr(class_object, self.method_post[1])(**form.cleaned_data)
            else:
                form_obj = form.save()
                result = self.msg(20001)
                if hasattr(self, 'return_field'):
                    if isinstance(self.return_field, tuple):
                        data = {}
                        for field in self.return_field:
                            data[field] = getattr(form_obj, field, '不存在')
                        result["data"] = data
                    else:
                        raise Exception("return_field类型为tuple")
            return JsonResponse(result)
        return JsonResponse(self.msg(list(form.errors.values())[0][0]))

    def put(self, request, *args, **kwargs):
        if not hasattr(request, 'data'):
            request.data = dict(urllib.parse.parse_qsl(request.body.decode()))
        if request.data.get('uuid') and hasattr(self, 'model_class') and self.model_class:
            try:
                object_one = self.model_class.objects.get(uuid=request.data.get('uuid'), del_state=1)
                form = self.form_class(request.data.copy(), instance=object_one)
                if form.is_valid():
                    if hasattr(self, 'method_put') and len(self.method_put) == 2:
                        method = self.method_put[0].split('.')
                        method_class = '.'.join(method[:-1])
                        action_class = getattr(importlib.import_module(method_class), method[-1])
                        class_object = action_class()
                        result = getattr(class_object, self.method_put[1])(request, form, *args, **kwargs)
                    else:
                        form_obj = form.save()
                        result = self.msg(20002)
                        if hasattr(self, 'return_field'):
                            # 自定义返回字段信息
                            if isinstance(self.return_field, tuple):
                                data = {}
                                for field in self.return_field:
                                    data[field] = getattr(form_obj, field, '不存在')
                                result["data"] = data
                            else:
                                raise Exception("return_field类型为tuple")
                        return JsonResponse(result)
            except:
                return JsonResponse(self.msg(50002))
        return JsonResponse(self.msg(list(form.errors.values())[0][0]))

    def delete(self, request, *args, **kwargs):
        if request.GET.get('uuid') and hasattr(self, 'model_class') and self.model_class:
            self.model_class.objects.filter(uuid=request.GET.get('uuid', '')).update(del_state=0)
            return JsonResponse(self.msg(20006))
        return JsonResponse(self.msg(50006))



from rest_framework import generics
from rest_framework.views import APIView
from django.http.response import HttpResponseBase
from rest_framework.response import Response
from django.utils.cache import cc_delim_re, patch_vary_headers
from rest_framework import exceptions, status
from djtool import Common


class APIView(APIView):
    pass


class GenericAPIView(generics.GenericAPIView, APIView, Common):

    transform_code = {
        '500': '50000',
        '405': '50005',
        '404': '50005',
        '403': '50000',
        '400': '50000',
        '200': '20000',
        '201': '20000',
    }

    def _validate_code(self, request, code):
        if isinstance(self.code, (str, int)):
            return self.code
        elif isinstance(self.code, dict):
            method = request.method.lower()
            if method in self.code:
                return self.code[method]
            else:
                raise Exception('请配置响应码（code）')
        else:
            raise Exception('配置响应码（code）格式错误')

    def _code(self, request, response, key='code'):
        if hasattr(self, key):
            return self._validate_code(request, getattr(self, key))
        elif hasattr(request, key):
            return self._validate_code(request, getattr(request, key))
        else:
            return self._transform(str(response.status_code))

    def _transform(self, status):
        try:
            return self.transform_code[status]
        except:
            raise Exception('请配置transform_code')

    def factory_response(self, request, response, *args, **kwargs):
        if 'code' not in response.data or 'msg' not in response.data:
            if response.exception:
                response.data = self.msg(self._code(request, response, key="code_fail"))
            else:
                response.data = self.msg(self._code(request, response), response.data)
        return response

    def finalize_response(self, request, response, *args, **kwargs):
        assert isinstance(response, HttpResponseBase), (
            'Expected a `Response`, `HttpResponse` or `HttpStreamingResponse` '
            'to be returned from the view, but received a `%s`'
            % type(response)
        )

        if isinstance(response, Response):
            if not getattr(request, 'accepted_renderer', None):
                neg = self.perform_content_negotiation(request, force=True)
                request.accepted_renderer, request.accepted_media_type = neg

            response.accepted_renderer = request.accepted_renderer
            response.accepted_media_type = request.accepted_media_type
            response.renderer_context = self.get_renderer_context()

        vary_headers = self.headers.pop('Vary', None)
        if vary_headers is not None:
            patch_vary_headers(response, cc_delim_re.split(vary_headers))

        for key, value in self.headers.items():
            response[key] = value

        response = self.factory_response(request, response, *args, **kwargs)

        return response

    def handle_exception(self, exc):
        if isinstance(exc, (exceptions.NotAuthenticated,
                            exceptions.AuthenticationFailed)):
            auth_header = self.get_authenticate_header(self.request)

            if auth_header:
                exc.auth_header = auth_header
            else:
                exc.status_code = status.HTTP_403_FORBIDDEN

        exception_handler = self.get_exception_handler()

        context = self.get_exception_handler_context()
        response = exception_handler(exc, context)

        if response is None:
            self.raise_uncaught_exception(exc)

        response.exception = True
        return response


class ListAPIView(GenericAPIView, generics.ListAPIView):
    pass


class CreateAPIView(GenericAPIView, generics.CreateAPIView):
    pass


class UpdateAPIView(GenericAPIView, generics.UpdateAPIView):
    pass


class RetrieveAPIView(GenericAPIView, generics.RetrieveAPIView):
    pass


class RetrieveUpdateAPIView(GenericAPIView, generics.RetrieveUpdateAPIView):
    pass



from __future__ import unicode_literals
import warnings


class RemovedInNextVersionWarning(DeprecationWarning):
    pass


def remove_check(**kwargs):
    deep = kwargs.get('deep', None)
    if deep is not None:
        warnings.warn('Parameter "deep" will removed in next version!', RemovedInNextVersionWarning, stacklevel=2)
        return deep
    return None



import os
from setuptools import setup, find_packages

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-tool',
    version='1.6.20',
    packages=find_packages(),
    include_package_data=True,
    license='BSD',
    description='A simple Django app to conduct Web-based tool.',
    long_description=README,
    url='https://pypi.python.org/pypi?:action=display&name=django-tool',
    author='Tsingliu',
    author_email='410869548@qq.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)


from django.db import models
import shortuuid
from djtool import Common


def createuuid():
    return shortuuid.uuid()


class BaseSimple(models.Model):
    uuid = models.CharField(
        'ID',
        max_length=22,
        primary_key=True,
        default=createuuid,
        editable=False)

    class Meta:
        abstract = True


class BaseMiddle(BaseSimple):
    add_time = models.DateTimeField('创建时间', auto_now_add=True)

    class Meta:
        abstract = True
        ordering = ['-add_time']

    def add_time_local(self):
        return Common.utcToLocal(self.add_time).strftime('%F %T')


class Base(BaseMiddle):
    del_state_type = ((0, '已删除'), (1, '默认'))
    modified_time = models.DateTimeField('修改时间', auto_now=True)
    del_state = models.IntegerField(
        '删除状态', choices=del_state_type, default=1, db_index=True)

    class Meta:
        abstract = True


class AdminBase(Base):
    unionuuid = models.CharField('统一用户ID', max_length=22)
    showname = models.CharField('姓名', max_length=50)

    class Meta:
        abstract = True

    def __str__(self):
        return self.showname



import re
import pytz
from datetime import datetime, date
from djtool.msgcode import tips
from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage
from django.utils.timezone import utc
from django.utils import timezone
import importlib


try:
    from common.msgcode import tips as tip
    if not isinstance(tip, dict):
        tip = {}
except:
    tip = {}


class Common:

    @classmethod
    def _tips(cls, code):
        if isinstance(code, int):
            code = str(code)
        if code:
            if code in tips:
                return tips[code]
            elif code in tip:
                return tip[code]
            raise Exception('msg对应编码不存在')
        raise Exception('msg参数类型错误')

    @classmethod
    def msg(cls, code, data=None, **kwargs):
        result = {}
        result['code'] = int(code)
        result['msg'] = cls._tips(code) if 'remsg' not in kwargs else kwargs.get('remsg', '')
        if data is not None:
            result['data'] = data
        return result

    @classmethod
    def mobile(cls, no):
        if isinstance(no, int):
            no = str(no)
        if isinstance(no, str):
            no = no.strip()
            pattern = re.compile(
                '^(0|86|17951)?(13[0-9]|15[012356789]|18[0-9]|14[57]|17[0-9])[0-9]{8}$'
            )
            if pattern.match(no):
                return pattern.match(no).group()
            return None
        return None

    @classmethod
    def email(cls, email):
        if isinstance(email, str):
            email = email.strip()
            pattern = re.compile(
                '^\w+((-\w+)|(\.\w+))*\@[A-Za-z0-9]+((\.|-)[A-Za-z0-9]+)*\.[A-Za-z0-9]+$'
            )
            if pattern.match(email):
                return pattern.match(email).group()
            return None
        return None

    @classmethod
    def add_set(cls, a, b):
        if isinstance(a, list) and isinstance(b, list):
            return list(set(a) - set(b))
        return None

    @classmethod
    def del_set(cls, a, b):
        if isinstance(a, list) and isinstance(b, list):
            return list(set(b) - set(a))
        return None

    @classmethod
    def list_toInt(cls, a):
        b = []
        for i in a:
            if isinstance(i, int):
                b.append(i)
            else:
                b.append(int(i))
        return b

    @classmethod
    def page(cls, res, pg, **kwargs):
        paginator = Paginator(res, kwargs.get('pre', 10))
        try:
            page = paginator.page(pg)
        except PageNotAnInteger:
            page = paginator.page(1)
        except EmptyPage:
            page = paginator.page(paginator.num_pages)
        return page

    @classmethod
    def utcToLocal(cls, t):
        return t.replace(
            tzinfo=pytz.utc).astimezone(pytz.timezone('Asia/Shanghai'))

    @classmethod
    def localToUtc(cls, str):
        tz = pytz.timezone('Asia/Shanghai')
        a = tz.localize(datetime.strptime(str, '%Y-%m-%d %H:%M:%S')).astimezone(pytz.utc)
        return a

    @classmethod
    def localUtc(cls):
        return timezone.now().replace(tzinfo=utc)

    @classmethod
    def ID(cls, pg, i, **kwargs):
        return (int(pg) - 1) * kwargs.get('pre', 10) + i

    @classmethod
    def ID_desc(cls, total, pg, i, **kwargs):
        return total - ((int(pg) - 1) * kwargs.get('pre', 10) + i) + 1

    @classmethod
    def year(cls, front=0, back=0, text=''):
        year = datetime.today().year
        l = [(y, '%s%s' % (y, text)) for y in range(year-front, year+back)]
        return tuple(l)

    @classmethod
    def expire(cls, d):
        today = date.today()
        if (d - today).days >= 0:
            return False
        return True

    @classmethod
    def import_model(cls, config):
        s = config.split('.')
        j = '.'.join(s[:-1])
        return getattr(importlib.import_module(j), s[-1])



# coding: utf-8

from __future__ import unicode_literals
from __future__ import absolute_import

import sys
PY2 = True
if sys.version < '3':
    from future.builtins import str, int
    PY2 = False

import json
from django.core.paginator import EmptyPage

from .Serializer import serializer
from .TimeFormatFactory import TimeFormatFactory

try:
    from django.http import HttpResponse
except ImportError:
    raise RuntimeError('django is required in django simple serializer')


class JsonResponseMixin(object):
    datetime_type = 'string'
    foreign = False
    many = False
    include_attr = []
    exclude_attr = []

    def time_format(self, time_obj):
        time_func = TimeFormatFactory.get_time_func(self.datetime_type)
        return time_func(time_obj)

    def context_serialize(self, context, *args, **kwargs):
        try:
            context.pop('view')
            context.pop('object')
        except KeyError:
            pass
        except AttributeError:
            pass
        # if kwargs.get('multi_extend'):
        #     self.include_attr.extend(kwargs.get('multi_extend'))
        return serializer(data=context,
                          datetime_format=self.datetime_type,
                          output_type='raw',
                          foreign=self.foreign,
                          many=self.many,
                          include_attr=self.include_attr,
                          exclude_attr=self.exclude_attr,
                          dict_check=True)

    @staticmethod
    def json_serializer(context):
        return json.dumps(context, indent=4)

    def render_to_response(self, context, **response_kwargs):
        context_dict = self.context_serialize(context)
        json_context = self.json_serializer(context_dict)
        return HttpResponse(json_context, content_type='application/json', **response_kwargs)


class FormJsonResponseMixin(JsonResponseMixin):
    def context_serialize(self, context, *args, **kwargs):
        form_list = []
        form = context.get('form', None)
        if form:
            for itm in form.fields:
                f_dict = {'field': str(itm)}
                form_list.append(f_dict)
        context_dict = super(FormJsonResponseMixin, self).context_serialize(context, *args, **kwargs)
        context_dict['form'] = form_list
        return context_dict


class MultipleJsonResponseMixin(JsonResponseMixin):
    def context_serialize(self, context, *args, **kwargs):
        # multi_extend = [i for i in context.keys() if not i.startswith('object') and i.endswith('_list')]
        # kwargs['multi_extend'] = multi_extend
        page_dict = {}
        is_paginated = context.get('is_paginated', None)
        if is_paginated:
            page_obj = context['page_obj']
            page_dict['current'] = page_obj.number
            page_dict['total'] = page_obj.paginator.num_pages
            try:
                previous_page = page_obj.previous_page_number()
            except EmptyPage:
                previous_page = None
            try:
                next_page = page_obj.next_page_number()
            except EmptyPage:
                next_page = None
            page_dict['previous'] = previous_page
            page_dict['next'] = next_page
            page_dict['page_range'] = [{'page': i} for i in page_obj.paginator.page_range]
        try:
            context.pop('paginator')
            context.pop('object_list')
        except KeyError:
            pass
        except AttributeError:
            pass
        context_dict = super(MultipleJsonResponseMixin, self).context_serialize(context, *args, **kwargs)
        context_dict['page_obj'] = page_dict
        return context_dict



from django.http import HttpResponseRedirect, JsonResponse
from django.core.cache import cache
from django.conf import settings
from djtool import ClientOauth, Common
oauth = ClientOauth()
try:
    assert hasattr(settings, 'ADMIN_MODEL')
    assert hasattr(settings, 'UNION_LOGIN_URL')
    Admin = Common.import_model(settings.ADMIN_MODEL)
except:
    raise Exception('请在settings.py配置ADMIN_MODEL, UNION_LOGIN_URL')


def adminlogin(fun):
    def new_fun(request, *args, **kwargs):
        url = request.path.split('/')
        if url[1] == "admin":
            uuid = request.session.get('login')
            try:
                if uuid:
                    admin = Admin.objects.get(uuid=uuid, del_state=1)
                    assert True if cache.get('admin%s' % admin.unionuuid) == 1 else False
                    request.admin = admin
                else:
                    token = request.COOKIES.get('_login')
                    if token:
                        info = oauth.info(token)
                        if info:
                            admin = Admin.objects.filter(unionuuid=info.get('uuid'), del_state=1)
                            if admin:
                                admin = admin[0]
                            else:
                                admin = Admin.objects.create(unionuuid=info.get('uuid'), showname=info.get('name', ''))
                            request.session['login'] = admin.uuid
                            request.session['info'] = info
                            request.admin = admin
                        else:
                            assert False
                    else:
                        assert False
            except Exception as e:
                print(e)
                if request.session.get('login'):
                    del request.session['login']
                if request.is_ajax():
                    return JsonResponse(Common.msg(61000))
                return HttpResponseRedirect('%s?backurl=%s' % (settings.UNION_LOGIN_URL, 'http://%s%s' % (request.get_host(), request.get_full_path())))
        return fun(request, *args, **kwargs)
    return new_fun
